from .Framenet17InferenceLoader import Framenet17InferenceLoader
from .Framenet17TrainingLoader import Framenet17TrainingLoader


__all__ = [
    "Framenet17InferenceLoader",
    "Framenet17TrainingLoader",
]
